import { precise_round } from '../utils/precise_round';

export default defineNuxtRouteMiddleware(async (to, from) => {
	//if (process.client) return; // skip middleware on server
	//if (process.server) return; // skip middleware on client side entirely

	let engine = {
		forceDevReload: false, // put dev environment
		benchmark: true,
		benchmark_start: null,
		benchmark_end: null,
		benchmarkElapsedTime: null,
	};

	if (engine.benchmark) engine.benchmark_start = performance.now();

	const nuxtApp = useNuxtApp();
	if (process.client && nuxtApp.isHydrating && nuxtApp.payload.serverRendered) return;

	/*
    // skip middleware on server
    if (process.server) return
    // skip middleware on client side entirely
    if (process.client) return
    // or only skip middleware on initial client load
    const nuxtApp = useNuxtApp()
    if (process.client && nuxtApp.isHydrating && nuxtApp.payload.serverRendered) return
    */

	// isAuthenticated() is an example method verifying if a user is authenticated
	/*
  if (isAuthenticated() === false) {
    return navigateTo('/login')
  }
  */

	const dynamicStore = useDynamicStore();
	console.log('route [MIDDLEWARE] - dynamicStore - context ', dynamicStore);

  const appConfig = useAppConfig();
  console.log('route [MIDDLEWARE] - _____useAppConfig____', appConfig);

// load shared app context including app && theme config
const appContext = ref<any | null>(null);
if (dynamicStore.appContext !== null) {
  appContext.value = dynamicStore.appContext; // or use appContext.value ?
  //appContext = dynamicStore.appContext; // or use appContext.value ?
  console.log('route [MIDDLEWARE] - 📘 [dynamicStore] read appContext', appContext.value);
} else {
  // the key didn't protect to redo the network request
  let keyBuilderAppContext = 'app_context';
  const { data: appContextFetch }: { data: any } = await useFetch('/api/app', { key: keyBuilderAppContext, method: 'post' });
  appContext.value = appContextFetch.value; // due to const dynamicDataFetch
  console.warn('route [MIDDLEWARE] - dynamic useFetch - appContext '+keyBuilderAppContext, appContextFetch.value, appContext.value);
  dynamicStore.setAppContext(appContext);
}

if (process.client && !appConfig.hasOwnProperty('appContext')) {
  updateAppConfig({ appContext: unref(appContext) });
  console.error('route [MIDDLEWARE] - _____useAppConfig____ write');
}

	// Get builder page data (slug(route.path) /settings/overview becomes settings_overview)
	// @todo add hostname + language + possible project reference
	let keySlug = to.path.indexOf('/') === 0 ? to.path.substr(1) : to.path;
	if (keySlug.indexOf('/') > 0) keySlug = keySlug.replace(/\//, '_');
	//if (keySlug.indexOf('admin_') === 0) keySlug = keySlug.replace('admin_', '');
	// debug: window.__NUXT__.data.dynamic_settings_overview
	let keyBuilderPageData = 'dynamic_' + keySlug;

	// no reactive dynamicData below
	//const dynamicFetchOptions: any = { method: 'post', body: { test: 1, path: route.path, slug: route.params.slug } };
	//const dynamicData = await $fetch('/api/dynamic?params1=a&param2=b', dynamicFetchOptions);

	// https://vueuse.org/core/useFetch/
	// @todo set default with skeleton localStorage code & lazy= true
	const dynamicFetchOptions: any = {
		key: keyBuilderPageData,
		method: 'post',
		//lazy: true,
		body: {
			test: 1,
			path: to.path,
			slug: to.params.slug,
			userAttributes: {
				urlPath: to.path,
			},
		},
		//pick: ['title']
	};

	// if we are using useFetch, the data returned is reactive so inside the script setup : dynamicData.value and inside template : dynamicData
	//const { data, pending, error, refresh } = await useFetch('https://api.nuxtjs.dev/mountains',{  pick: ['title'] });
	//'/api/dynamic?params1=a&param2=b'

	const dynamicData = ref<any | null>(null);
	console.warn('route [MIDDLEWARE] ____hasPageByKey___ ' + keySlug, dynamicStore.hasPageByKey(keySlug));
	if (dynamicStore.hasPageByKey(keySlug)) {
		dynamicData.value = dynamicStore.getPageByKey(keySlug); // or use appContext.value ?
		console.log('route [MIDDLEWARE] - 📘 [dynamicStore] read page ' + keySlug, dynamicData.value);
	} else {
		console.log('route [MIDDLEWARE] - before fetch ' + keyBuilderPageData);

		//const { data: dynamicData, pending, error }: { data: any, pending: boolean, error?: any} = await useFetch('/api/dynamic', dynamicFetchOptions);
		//console.warn('dynamic [CLIENT] - dynamicData '+keyBuilderPageData, dynamicData.value);

		const { data: dynamicDataFetch,	pending, error }: { data: any; pending: boolean; error?: any } = await useFetch('/api/dynamic', dynamicFetchOptions);
		dynamicData.value = dynamicDataFetch.value; // due to const dynamicDataFetch
		console.warn('route [MIDDLEWARE] - dynamic useFetch - dynamicData ' + keyBuilderPageData, dynamicData.value);

		// auto redirection if not found
		if (dynamicData.value && dynamicData.value.hasOwnProperty('page') && dynamicData.value.page === null) {
			if (dynamicData.value?._metadata?.engine && dynamicData.value._metadata.engine.notFoundRedirection) {
				console.warn('route [MIDDLEWARE] - redirection - navigateTo ' + dynamicData.value._metadata.engine.notFoundRedirection);
				return navigateTo(dynamicData.value._metadata.engine.notFoundRedirection);
			} else {
				console.warn('route [MIDDLEWARE] - redirection - disabled');
			}
		} else {
			// save valid page data on dynamic store
			dynamicStore.setPageContext(keySlug, dynamicData);
		}
	}

	let customlayout = 'blank';
	if (dynamicData.value?.page?.layout?.template) customlayout = dynamicData.value.page.layout.template;

	// Set the layout on the route you are navigating _to_
	setPageLayout(customlayout);
	// setPageLayout('settings');
	to.meta.layout = customlayout; // erase meta directly

	/*
   const layout = useState('layout');
   if (!layout.value) {
     // This won't trigger on client side
     console.log('fetching layout for page')
     await new Promise((resolve) => setTimeout(resolve, 100));
     layout.value = 'other';
   }
   to.meta.layout = layout.value;
   */

	//console.log('Strictly forbidden.');
	//return false;

	//const route = useRoute();
	//console.log('__middleware__', route.meta);
	//let page_dynamic_config = to?.meta?.dynamic && Object.freeze(to.meta.dynamic) || {};
	let page_dynamic_config = (to?.meta?.dynamic && { ...to.meta.dynamic, page: dynamicData.value?.page || null }) || {};
	// @ts-ignore
	page_dynamic_config.fullPath = to.fullPath;
	console.log('route [MIDDLEWARE] - route', to, to.meta, from.meta, JSON.stringify(page_dynamic_config), dynamicData.value?.page); // to.params.story

	// @ts-ignore
	if (to.params.story == 'onboarding') page_dynamic_config.navbar = false;
	//page_dynamic_config.navbar = (to.params.story != 'onboarding') ? true : false;

	//const appConfig = useAppConfig(); // { foo: 'bar' }
	//const newAppConfig = { dynamic: page_dynamic_config };
	const newAppConfig = { foo: 'baz', dynamic: page_dynamic_config };
	updateAppConfig(newAppConfig);

	if (engine.benchmark) {
		engine.benchmark_end = performance.now();
		engine.benchmarkElapsedTime = precise_round(engine.benchmark_end - engine.benchmark_start, 2);
		console.log('route [MIDDLEWARE] - benchmark getConfiguration ' + engine.benchmarkElapsedTime + ' ms');
	}
});
