//import { navigateTo } from '#imports';
import { precise_round } from '../../utils/precise_round';

// @todo replace console.log with external pino logger
export default defineEventHandler(async (event) => {
	console.log('dynamic [SERVER] - new request: ' + event.node.req.url);
	//console.log('dynamic [SERVER] - context', event.context);

	// init metadata & engine context
	// @todo Engine rules (it should overwrite from ENV)
	let _metadata = {
		engine: {
			debug: true,
			forceDevReload: true, // put false in production
			notFoundRaiseError: false, // redirection should be managed on client side
			notFoundRedirection: false, // '/', // or false to disable auto redirection
      benchmark: true,
		},
		cache: {
			tag: null,
			key: null,
			adapter: null, // null, memory, file, redis
			save: false,
			elepased_time: null,
			error: null,
		},
	};

  let benchmark = {
    benchmarkStart: null,
    benchmarkEnd: null,
    benchmarkElapsedTime: null,
  }

	if (_metadata.engine.benchmark) benchmark.benchmarkStart = performance.now();

	const query = getQuery(event);
	console.log('dynamic [SERVER] - query', query);

	const body = await readBody(event);

	//let content = null;
	//content = await fetchContent(filePath);
	//content = await $fetch(filePath);
	//content =  require(`../data/reports.json`);

	/*
    jsons = ["json_one","json_two"]
	jsons_readed = []

	// In the loop
	file = require(`./assets/data/geo/${jsons[i]}`)
	jsons_readed.push(file)
    */

	let slugFileName = '';
	let validFilePaths = [];
	// body.path=/onboarding/persona
	if (body && body.path) {
		slugFileName = body.path.indexOf('/') === 0 ? body.path.substr(1) : body.path;
		validFilePaths.push(slugFileName); // onboarding/persona
		if (slugFileName.indexOf('/') > 0) validFilePaths.push(slugFileName.replace(/\//, '_')); // searching onboarding_persona
	}

	// get last slug part object
	// @todo use first to get layout like settings
	/*
	if (body?.slug && body.slug.length > 0) {
		slugFileName = body.slug.slice(-1)[0];
	}
	*/
	console.log('dynamic [SERVER] - slugFileName=' + slugFileName);
	//console.log('dynamic [SERVER] - validFilePaths', validFilePaths);

	let page_config = null;
	if (slugFileName != '' || validFilePaths.length > 0) {
		// init slug key in memory
		// @toddo use md5 hash with params & per user
		let cacheSlugKey = 'cache:slug_' + slugFileName.replace(/\//, '');
		let cachePageConfigExist = await useStorage().hasItem(cacheSlugKey);

		_metadata.cache.tag = slugFileName.replace(/\//, '_');
		_metadata.cache.key = cacheSlugKey;

		// get cache slug key in memory
		if (cachePageConfigExist && !_metadata.engine.forceDevReload) {
			page_config = await useStorage().getItem(cacheSlugKey);
			_metadata.cache.adapter = 'memory';
		} else {
			// @todo do stuff with alias

			try {
				// @todo list
				// 1. make in memory first or shared redis, then load local file failback (cf https://github.com/unjs/unstorage)
				// 2. Call direct remote API to retrieve dynamic json
				// 3. server middleware to verify authentification, secured page & permissions

				// Single entry point
				//let filePagePath = `assets/server/logic/${slugFileName}.json`;
				//page_config = await useStorage().getItem(filePagePath);

				// Multiple entries point in FIFO
				for (const slugFilePath of validFilePaths) {
					let filePagePath = `assets/server/logic/${slugFilePath}.json`;
					page_config = await useStorage().getItem(filePagePath);
					if (!page_config) {
						// failback in data directory
						let filePagePath = `assets/server/data/${slugFilePath}.json`;
						page_config = await useStorage().getItem(filePagePath);
					} else {
						break;
					}
				}

				_metadata.cache.adapter = 'file';

				if (!page_config) {
					_metadata.cache.error = 'notfound';
					console.log('dynamic [SERVER] - redirection');
					// raise fatal 404 or other strategy
					if (_metadata.engine.notFoundRaiseError) throw createError({ statusCode: 404, statusMessage: 'Page Not Found' });
					//else if (_metadata.engine.notFoundRedirection) return navigateTo(_metadata.engine.notFoundRedirection); // or await
				} else {
					console.log('dynamic [SERVER] - write cache for page config');
					await useStorage().setItem(cacheSlugKey, page_config);
					_metadata.cache.save = true;
				}
			} catch (err) {
				console.error('dynamic [SERVER] - error', err);
				//throw createError({ statusCode: 404, statusMessage: 'Page Not Found' })
			}
		}
	}

  if (_metadata.engine.benchmark) {
    benchmark.benchmarkEnd = performance.now();
    benchmark.benchmarkElapsedTime = precise_round(benchmark.benchmarkEnd - benchmark.benchmarkStart, 2);
    _metadata.cache.elepased_time = benchmark.benchmarkElapsedTime;
    console.log('dynamic [SERVER] - benchmark getConfiguration ' + benchmark.benchmarkElapsedTime + ' ms');
  }

	return { page: page_config, body, query, _metadata };
});
