//import { defineStore } from 'pinia'; // no need with auto import on nuxt-dynamic-layer
//import axios from 'axios';
import jwtDecode from "jwt-decode";
//import { USER_ROLE } from '@/constants';
//import accountRoleEnum from '@/enums/accountRoleEnum';
//import { createToast } from 'mosha-vue-toastify';

//import { directus } from '@/services/directus';
// const { $directus } = useNuxtApp();

//https://github.com/vuejs/pinia/discussions/1513 => store pinia with useNuxtApp

const accountRoleEnum = Object.freeze({
    ADMIN: "59a02bb1-a573-4d8b-b0d9-3e381b7b3041", // administrateur
    TECHNICIAN: "7c720408-86ae-472c-83bc-1b9a0c947267", // Préparateur
    CUSTOMER: "b6d6c5cc-6e21-4907-814d-691909c07da6", // Comité
    //ADMIN: "admin",
    //TECHNICIAN: "technician",
    //TECHNICIAN_MANAGER: "technicianManager",
    //WORKSHOP_MANAGER: "workshopManager",
    //CUSTOMER: "customer"
});

const token = typeof localStorage !== 'undefined' && localStorage.getItem('auth_token') || null; // retrieve access_token
const user_info = (typeof localStorage !== 'undefined' && localStorage.getItem('user_info') && JSON.parse(localStorage.getItem('user_info'))) || null; // retrieve user_info

/*
 access_token decoded :
    admin_access: 0
    app_access: 1
    exp: 1649253031
    iat: 1649252131
    id: "73092cb0-443f-42dd-bc7c-1a3bf7cc61dd"
    iss: "directus"
    role: "7c720408-86ae-472c-83bc-1b9a0c947267"
*/
/*
function getUser(token) {
	var tokenDecoded = token ? jwtDecode(token) : null;
	//console.log('getUser', tokenDecoded);
	return tokenDecoded
		? {
				id: tokenDecoded.id,
				//firstname: tokenDecoded.first_name,
				//lastname: tokenDecoded.last_Sname,
				role: tokenDecoded.role,
		  }
		: null;
}
*/

// But, we need to authenticate if data is private
//let authenticated = false;

/*
// error: [nuxt] A composable that requires access to the Nuxt instance was called outside of a plugin, Nuxt hook, Nuxt middleware, or Vue setup function. This is probably not a Nuxt bug. Find out more at https://nuxt.com/docs/guide/concepts/auto-imports#using-vue-and-nuxt-composables.
const config = useRuntimeConfig();
console.log('auth config', config);

if (process.client) console.log('auth didi client', config);
else if (process.server) console.log('auth didi server', config);

console.log('auth import env', import.meta, process.env);

// error: [nuxt] A composable that requires access to the Nuxt instance was called outside of a plugin, Nuxt hook, Nuxt middleware, or Vue setup function. This is probably not a Nuxt bug. Find out more at https://nuxt.com/docs/guide/concepts/auto-imports#using-vue-and-nuxt-composables.
const app = useAppConfig();
console.log('auth app', app);

const engine_config = config?.public?.authentification || {}; // if (config?.public?.authentification?.withAxios);
*/
const engine_config = {};

const trace_debug = import.meta.env.DEV;

//console.log(app);

export const useAuthentificationStore = defineStore('authentification', {
	//id: 'authentificationStore',
	// initial state
	state: () => ({
		token: token,
		user: user_info,
		//user: getUser(token)
	}),
	getters: {
		isMobile: () => {
			if (process.client && typeof window !== 'undefined' && typeof localStorage !== 'undefined') return localStorage['isFromAppTablet'] !== 'true' && window.screen.width <= 768;
			else return false; // from ssr server
		},
		isFromAppTablet: (state) => {
			if (process.client && typeof window !== 'undefined' && typeof localStorage !== 'undefined') {
				if (import.meta.env.DEV && window.screen.width <= 960 && localStorage['isFromAppTablet'] === 'true') return true;
				else if (!import.meta.env.DEV && import.meta.env.PROD && window.screen.width == 960) return true; // for debug tablet on production
				else if (import.meta.env.DEV && localStorage['isFromAppTablet'] === 'true') return false;
				else return localStorage['isFromAppTablet'] === 'true';
			} else {
				return false; // from ssr server
			}
		},
		isAuthenticated: (state) => {
			//return true; // for dev
			return state.token !== null;
		},
		hasRole: (state) => (role) => {
			return state.user?.role === role;
		},
		hasRoleInList: (state) => (roles) => {
			for (let role of roles) {
				if (this.hasRole(role)) {
					return true;
				}
			}
			return false;
		},
		isAdmin: (state) => {
			//return true; // for dev
			return state.user?.role === accountRoleEnum.ADMIN;
		},
		// current user id
		getUserId: (state) => {
			return state.user?.id || false;
		},
		getUserExternalIdentifier: (state) => {
			return state.user?.external_identifier || false;
		},	
	},
	actions: {
		// AUTHENTICATION_REFRESH
		async refreshToken() {
			const { $directus } = useNuxtApp();
			
			// Try to authenticate with token if exists
			await $directus.auth
				.refresh()
				.then((response) => {
					//authenticated = true;
					if (trace_debug) console.warn('___authentificationStore - AUTHENTICATION_REFRESH success');
					this.updateToken({ token: response.access_token, user_info: context.state.user_info });
				})
				.catch(() => {
					if (trace_debug) console.warn('___authentificationStore - AUTHENTICATION_REFRESH failed');
					this.logout();
				});
		},

		// AUTHENTICATION_LOGIN : account {email && password, or optional tag (by using tag as static token used on NFC badge) }
		login(account) {
			var self = this;
					
			const { $directus } = useNuxtApp();

			return new Promise(async (resolve, reject) => {
				if (trace_debug) console.log('___authentificationStore - login', account);
							
				if (account?.email && account.email.indexOf('@') === -1) {
					//self.isFromAppTablet
					if (trace_debug) console.log('___authentificationStore - login by firstname/username', account);
			
					let queryOptions = {
						fields: "email, first_name, last_name, status, role",
						filter: {
							first_name: {
								_eq: account.email
							},
							role: {
								_in: [accountRoleEnum.TECHNICIAN, accountRoleEnum.ADMIN]
							},
							status: {
								_eq: 'active'
							}
						},
						sort: 'first_name',
						limit: -1,
					};
					const responseSearchUserByFirstName = await $directus.items('directus_users').readByQuery(queryOptions).catch((err) => {
						if (trace_debug) console.error('___authentificationStore - error api responseSearchUserByFirstName', err);
						//window.alert(err.message);
					});

					if (responseSearchUserByFirstName && responseSearchUserByFirstName.data && responseSearchUserByFirstName.data.length > 0) {
						account.email = responseSearchUserByFirstName.data[0].email;
						if (trace_debug) console.log('___authentificationStore - search user', account, responseSearchUserByFirstName.data);
					}
				}

				/*
				axios.post('account/authenticate', account, {
						toastError: true,
						showPreloader: true
					})
				*/
				
				/*
				axios.post('account/authenticateByTag', tag, {
					toastError: true,
					showPreloader: true
				})
				*/
				
				/* 
				{
					// Temporary access token to be used in follow-up requests.
					access_token: '...',

					// The token that can be used to retrieve a new access token through /auth/refresh. Note: if you used cookie as the mode in the request, the refresh token won't be returned in the JSON.
					refresh_token: undefined,

					// How long before the access token will expire. Value is in milliseconds.
					expires: 900000 // default is 15min => see ACCESS_TOKEN_TTL
				}
				*/			
				try {
					let response = null;
					
					if (account?.tag) {
						// use tag as static token
						response = await $directus.auth.static(account.tag);
					} else {
						response = await $directus.auth.login(account);
					}
						
					if (trace_debug) console.warn('___authentificationStore - api response', response);
					
					if (response !== null) {
						let access_token = response.access_token;
						//localStorage.setItem('token', access_token);
													
						if (engine_config?.withAxios) axios.defaults.headers.common['Authorization'] = `Bearer ${access_token}`;

						let user_info = await $directus.users.me.read({
							fields: [
								'id',
								'first_name',
								'last_name',
								'email',
								'avatar',
								'tags',
								'role',
								'language',
								'last_access',
								'last_page',
								'provider',
								'external_identifier',
								'committee.*', // @todo add optional custom fields here
							],
						});
						if (trace_debug) console.log('___authentificationStore - api user_info', user_info);
						if (typeof localStorage !== 'undefined') localStorage.setItem('user_info', JSON.stringify(user_info));

						self.updateToken({ token: access_token, user_info: user_info });

						// @todo à vérifier pour préparateur
						// On vérifie qu'un agent n'essaye pas de se connecter sur pc
						/*
						if(this.isTechnician && !this.isFromAppTablet) {
							this.logout()
							createToast(`Vous n'avez pas accès à ce site. Veuillez utiliser l'application tablette.`, {
								position: 'top-right',
								type: 'danger',
								hideProgressBar: true,
								showIcon: true,
								transition: 'slide'
							});
						}
						*/
						resolve(true);
					} else {
						self.updateToken(null);
						reject();
					}
				} catch(e) {
					if (trace_debug) console.error('___authentificationStore - error', e);
					//localStorage.removeItem('token');
					if (typeof localStorage !== 'undefined') {
						localStorage.removeItem('auth_token');
						localStorage.removeItem('auth_refresh_token');
						localStorage.removeItem('auth_expires_at');
						localStorage.removeItem('auth_expires');
						localStorage.removeItem('user_info');
					}
					if (engine_config?.withAxios) delete axios.defaults.headers.common['Authorization'];
					self.updateToken(null);
					reject();
					//window.alert('Invalid credentials');
				};
				
			});
		},

		// AUTHENTICATION_LOGOUT
		async logout() {
			const { $directus } = useNuxtApp();

			//localStorage.removeItem('token');
			await $directus.auth.logout().catch((err) => {
				if (trace_debug) console.error('___authentificationStore - logout - api data error', err);
				//  Error: "refresh_token" is required in either the JSON payload or Cookie
			});
			
			if (typeof localStorage !== 'undefined') {
				localStorage.removeItem('auth_token');
				localStorage.removeItem('auth_refresh_token');
				localStorage.removeItem('auth_expires_at');
				localStorage.removeItem('auth_expires');
				localStorage.removeItem('user_info');
			}
			if (engine_config?.withAxios) delete axios.defaults.headers.common['Authorization'];

			this.updateToken(null);
			
			// put variable below
			//document.location.href = '/connexion';
		},

		// AUTHENTICATION_TOKEN_CHANGE
		updateToken(payload) {
			if (trace_debug) console.log('___authentificationStore - updateToken', payload);
			this.token = payload ? payload.token : null;
			this.user = payload ? payload.user_info : null; //getUser(token);
		},
	},
	hydrate(state, initialState) {
		// https://pinia.vuejs.org/cookbook/composables.html
		// in this case we can completely ignore the initial state since we want to read the value from the browser
		//state.user = useLocalStorage('pinia/auth/login', 'bob')
		
		const config = useRuntimeConfig();
		console.log('auth config from hydrate', config);

		state.token = (typeof localStorage !== 'undefined' && localStorage.getItem('auth_token')) || null; // retrieve access_token
		state.user = (typeof localStorage !== 'undefined' && localStorage.getItem('user_info') && JSON.parse(localStorage.getItem('user_info'))) || null; // retrieve user_info

		console.warn('___authentificationStore - init hydrate', initialState, state, process.client);
		//state.counter = mainStoreInitialData.counter
	},	
	persist: {
		// store options goes here
		key: 'storeAuth',
		storage: persistedState.cookiesWithOptions({
			sameSite: 'strict',
		}),
	}	
});

if (import.meta.hot) {
  import.meta.hot.accept(acceptHMRUpdate(useAuthentificationStore, import.meta.hot))
}