// import { defineStore } from "pinia"; // no need with auto import on nuxt-dynamic-layer
//import { acceptHMRUpdateWithHydration } from '@pinia-plugin-persistedstate/hmr';
import { useAuthentificationStore } from './authentification';
//import { directus } from '@/services/directus';

/*
import { useApolloClient } from "@vue/apollo-composable";
import { cartCreate } from "~/apollo/mutations/cartCreate";
import { cartLinesAdd } from "~/apollo/mutations/cartLinesAdd";
import { cartLinesRemove } from "~/apollo/mutations/cartLinesRemove";
import { checkoutUrl } from "~/apollo/queries/checkoutUrl";
import { formatLocalePrice } from "~/utils/money";
*/
import { formatLocalePrice } from "../utils/money";

// inspired from https://github.com/vuejs/vuex/blob/3.x/examples/shopping-cart

const cart_id = typeof localStorage !== 'undefined' && localStorage.getItem('cart_id') || null;
const cart_status = typeof localStorage !== 'undefined' && localStorage.getItem('cart_status') || null;
const cart_items = (typeof localStorage !== 'undefined' && localStorage.getItem('cart_items') && JSON.parse(localStorage.getItem('cart_items'))) || [];

const cart_default_currency_code = "EUR"; // USD
const cart_default_locale = "fr-FR"; // en-US

export const useCartStore = defineStore("cart", {
  //id: 'cartStore',
  state: () => {
    return {
		items: cart_items, // [],
		checkoutStatus: cart_status, //null,
		checkoutId: cart_id, //null		
		
		// from NuxtShop
		cart: {
			checkoutUrl: null,
			estimatedCost: {
			  subtotalAmount: {
				amount: "",
				currencyCode: cart_default_currency_code, //"USD",
			  },
			  totalTaxAmount: {
				amount: "",
				currencyCode: cart_default_currency_code, //"USD",
			  },
			  totalDutyAmount: {
				amount: "",
				currencyCode: cart_default_currency_code, //"USD",
			  },
			  totalAmount: {
				amount: "",
				currencyCode: cart_default_currency_code, //"USD",
			  },
			},
			id: null, // the checkoutID = cartId
			lines: {
			  edges: [],
			},
		},
		loading: false, // current loading flag of the cart during server calls
		cartOpen: false, // current opening flag for cart to display on screen
    };
  },
  getters: {
	  	/*
		cartProducts: (state, getters, rootState) => {
		    return state.items.map(({ id, quantity }) => {
		      const product = rootState.products.all.find(product => product.id === id)
		      return {
		        id: product.id,
		        title: product.title,
		        price: product.price,
		        quantity
		      }
		    })
		},
		*/
		/*
		cartProducts: (state) => {
			return state.items;
			//return state.items.reverse();
		},
		*/
		cartTotalPrice: (state) => {
			console.warn(state.items);
			let t = state.items.reduce((total, product) => {
				return total + product.price * product.quantity;
			}, 0);
			return parseFloat(t).toFixed(2);
		},
		// refresh badge from shopping store
		totalProducts: (state) => {
			return state.items.length;
		},
		
		// from NuxtShop
		lineItems: (state) => state.cart.lines.edges,
		subtotalAmount: (state): string => {
		  const amount = +state.cart.estimatedCost.subtotalAmount.amount;
		  const code = state.cart.estimatedCost.subtotalAmount.currencyCode;
		  return formatLocalePrice(+amount, cart_default_locale, code);
		},
		totalDutyAmount: (state): string => {
		  if (!state?.cart?.estimatedCost?.totalDutyAmount) {
			return null;
		  }
		  const amount = +state?.cart?.estimatedCost?.totalDutyAmount?.amount;
		  const code = state?.cart?.estimatedCost?.totalDutyAmount?.currencyCode;
		  return formatLocalePrice(+amount, cart_default_locale, code);
		},
		totalTaxAmount: (state): string => {
		  if (!state?.cart?.estimatedCost?.totalTaxAmount) {
			return null;
		  }
		  const amount = +state.cart.estimatedCost.totalTaxAmount.amount;
		  const code = state.cart.estimatedCost.totalTaxAmount.currencyCode;
		  return formatLocalePrice(+amount, cart_default_locale, code);
		},
		totalAmount: (state): string => {
		  const amount = +state.cart.estimatedCost.totalAmount.amount;
		  const code = state.cart.estimatedCost.totalAmount.currencyCode;
		  return formatLocalePrice(+amount, cart_default_locale, code);
		},
	
  },
  actions: {
		// context has commit, state, getters & dispatch
		// deliveryInfo contains delivery_address && delivery_type
		async checkout(deliveryInfo) {
			//let cart_id = (typeof localStorage['cart_id'] !== 'undefined') ? localStorage['cart_id'] : false;

			if (this.checkoutId) {
				const savedCartItems = [...this.items];

				this.setCheckoutStatus(null);

				// empty cart
				this.setCartItems({
					items: [],
				});
				
				const { $directus } = useNuxtApp();

				// save in db
				const responseCheckoutOrder = await $directus
					.items('orders')
					.updateOne(this.checkoutId, {
						status: 'published',
						date_order: new Date(),
						//user_validated: self.getCurrentUserId,
						//date_validated: new Date(),
						delivery_type: deliveryInfo.delivery_type,
						delivery_address: deliveryInfo.delivery_address,
					})
					.then(() => {
						console.warn('success checkout');

						this.setCheckoutStatus('Commande envoyée avec succès.');

						this.setCheckoutId(null);
					})
					.catch((err) => {
						console.error('responseCheckoutOrder', err);

						this.setCheckoutStatus('Commande en échec.');
						// rollback to the cart saved before sending the request
						this.setCartItems({
							items: savedCartItems,
						});
					});
			}
		},
		/*
		checkout ({ commit, state }, products) {
		    const savedCartItems = [...state.items];
		    commit('setCheckoutStatus', null);
		    // empty cart
		    commit('setCartItems', { items: [] });

		    shop.buyProducts(
		      products,
		      () => commit('setCheckoutStatus', 'successful'),
		      () => {
		        commit('setCheckoutStatus', 'failed')
		        // rollback to the cart saved before sending the request
		        commit('setCartItems', { items: savedCartItems })
		      }
		    )
		},
		*/
		async addProductToCart(product, options) {
			console.log('store addProductToCart ', product, options);

			if (!options) options = {};
			if (!options.hasOwnProperty('refreshLive')) options.refreshLive = 'incremental'; // false, init (products view), incremental (cart) or original

			const authStore = useAuthentificationStore();
			
			const { $directus } = useNuxtApp();

			let committeeId = authStore.getCommitteeId;
			if (!committeeId) committeeId = 5; // only for dev from admin => need to comment in future
			console.log('committeeId=' + committeeId);

			let committeeFamilies = authStore.getCommitteeFamilies;
			if (!committeeFamilies) committeeFamilies = 0; // only for dev from admin => need to comment in future
			console.log('committeeId=' + committeeId + ' committeeFamilies=' + committeeFamilies);

			//if (!committeeId) return false; // disable from admin

			this.setCheckoutStatus(null);

			if (committeeId !== false && typeof localStorage['cart_id'] === 'undefined') {
				// if no local storage, create a empty order as draft shopping cart
				let formOrder = {
					status: 'draft',
					committee: committeeId,
					families: committeeFamilies,
					source: 'web',
				};

				const responseDraftOrder = await $directus
					.items('orders')
					.createOne(formOrder)
					.catch((err) => {
						console.error('responseDraftOrder', err);
					});

				console.log(responseDraftOrder);
				if (responseDraftOrder) {
					this.setCheckoutId(responseDraftOrder.id);
				}
			}

			let cart_id = typeof localStorage['cart_id'] !== 'undefined' ? localStorage['cart_id'] : false;

			if (cart_id) {
				//if (product.inventory > 0) {
				const cartItem = this.items.find((item) => item.id === product.id);
				if (!cartItem) {
					//commit('pushProductToCart', { id: product.id })
					this.pushProductToCart(product);
					//commit('pushProductToCart', { id: product.id, quantity: product.quantity });

					// save in db
					let formOrderProduct = {
						order: cart_id,
						product: product.id,
						quantity: product.quantity,
						//status: 'draft',
					};

					const responseDraftOrderProduct = await $directus
						.items('order_products')
						.createOne(formOrderProduct)
						.catch((err) => {
							console.error('responseDraftOrderProduct', err);
						});

					console.log(responseDraftOrderProduct);
				} else {
					// read order product line
					const orderProductData = await $directus
						.items('order_products')
						.readByQuery({
							filter: {
								order: {
									_eq: cart_id,
								},
								product: {
									_eq: product.id,
								},
							},
							limit: 1,
						})
						.catch((err) => {
							console.error('orderProductData', err);
						});

					const orderProductItem = orderProductData.data && orderProductData.data.length > 0 ? orderProductData.data[0] : null;
					console.log('orderProductItem', orderProductItem);

					if (orderProductItem && parseInt(product.quantity) > 0) {
						//let newQuantity = (options.refreshLive && options.refreshLive == 'init') ? ''+(parseInt(cartItem.quantity) + parseInt(product.quantity)) : product.quantity;
						let newQuantity = product.quantity;

						this.setItemQuantity({
							...cartItem,
							quantity: newQuantity
							//quantity: product.quantity,
						}, options.refreshLive);
						//commit('incrementItemQuantity', cartItem);

						// save in db
						const responseDraftOrderProduct = await $directus
							.items('order_products')
							.updateOne(orderProductItem.id, {
								order: cart_id,
								product: product.id,
								quantity: newQuantity
								//quantity: product.quantity,
								//status: 'draft',
							})
							.catch((err) => {
								console.error('responseDraftOrderProduct', err);
							});

						console.log(responseDraftOrderProduct);

						this.refreshStock();

					} else if (orderProductItem && parseInt(product.quantity) <= 0) {
						this.deleteProductFromCart(product.id);

						// save in db
						const responseDraftOrderProduct = await $directus
							.items('order_products')
							.deleteOne(orderProductItem.id)
							.catch((err) => {
								console.error('responseDraftOrderProduct', err);
							});

						console.log(responseDraftOrderProduct);
					}
					// remove 1 item from stock
					//commit('products/decrementProductInventory', { id: product.id }, { root: true })
					//}

					// @todo notify toast
				}
			} else {
			}
		},
		
		
		async refreshStock(saveLocalStorage = true) {
			console.log('refreshStock');
			if (this.items.length > 0) {
				
				const { $directus } = useNuxtApp();
				
				// -------
				// retrieve live stock (order_products.status == draft or prepared with order.status = draft, published, validated)
				// included the current committee shopping cart
				let pendingStockProducts = {};

				let queryStockPendingOptions = {
					fields: 'id, status, quantity, order.id, order.status, order.committee, product',
					filter: {
						status: {
							_in: ['draft', 'prepared'],
						},
						order: {
							status: {
								_in: ['draft', 'published', 'validated'],
							}
						}
					},
					aggregate: {
						sum: 'quantity'
					},
					sort: 'product',
					groupBy: ['product'],
					limit: -1,
				};

				const responseStockPending = await $directus
					.items('order_products')
					.readByQuery(queryStockPendingOptions)
					.catch((err) => {
						console.error('responseStockPending', err);
					});
				console.log('api responseStockPending', responseStockPending);

				if (responseStockPending && responseStockPending.data) {
					responseStockPending.data.forEach(function (pendingStock) {
						pendingStockProducts[pendingStock.product] = pendingStock.sum.quantity;
					});
					console.log('pendingStockProducts', pendingStockProducts);
				}
				// ------

				for (const element of this.items) {
					// @todo detect pending stock : order_products.status=='draft'
					let estimateStock = element.stock_quantity;
					if (estimateStock > 0 && pendingStockProducts[element.id]) {
						estimateStock = estimateStock - pendingStockProducts[element.id];
						if (estimateStock < 0) estimateStock = 0;
					}
					element.stock_quantity_live = estimateStock;

					element.stock_quantity_live_original = estimateStock;
				}

				console.warn(this.items);

				if (saveLocalStorage) localStorage.setItem('cart_items', JSON.stringify(this.items));
			}
		},

		// we need at least product: { id, quantity }
		pushProductToCart(product) {
			this.items.push(product);

			localStorage.setItem('cart_items', JSON.stringify(this.items));
		},

		/*
		pushProductToCart (state, { id, quantity }) {
		    state.items.push({
		      id,
		      quantity
		      //quantity: 1
		    });

		    localStorage.setItem('cart_items', JSON.stringify(state.items));
		},
		*/

		setItemQuantity({ id, quantity }, refreshLive = 'incremental') {
			const cartItem = this.items.find((item) => item.id === id);
			cartItem.quantity = quantity;

			/*
			const oldLive = cartItem.stock_quantity_live;

			if (cartItem.hasOwnProperty('stock_quantity_live')) {
				if (refreshLive && refreshLive == 'incremental') {
					cartItem.stock_quantity_live = cartItem.stock_quantity_live - parseInt(quantity);
				} else if (refreshLive && refreshLive == 'decremental') {
						cartItem.stock_quantity_live = cartItem.stock_quantity_live + parseInt(quantity);
				} else if (refreshLive && (refreshLive == 'original' || refreshLive == 'init') && cartItem.hasOwnProperty('stock_quantity_live_original')) {
					cartItem.stock_quantity_live = cartItem.stock_quantity_live_original - parseInt(quantity);
				}
				if (cartItem.stock_quantity_live < 0) cartItem.stock_quantity_live = 0;
			}
			*/

			console.warn('_____refreshLive='+refreshLive+' id='+id+' quantity='+quantity);
			//console.warn('_____refreshLive='+refreshLive+' quantity='+quantity+' oldLive='+oldLive+' newLive='+cartItem.stock_quantity_live);
			localStorage.setItem('cart_items', JSON.stringify(this.items));
		},

		incrementItemQuantity({ id }) {
			const cartItem = this.items.find((item) => item.id === id);
			cartItem.quantity++;

			if (cartItem.hasOwnProperty('stock_quantity_live')) {
				cartItem.stock_quantity_live = cartItem.stock_quantity_live--;
				if (cartItem.stock_quantity_live < 0) cartItem.stock_quantity_live = 0;
			}

			localStorage.setItem('cart_items', JSON.stringify(this.items));
		},

		decrementItemQuantity({ id }) {
			const cartItem = this.items.find((item) => item.id === id);
			cartItem.quantity--;

			if (cartItem.hasOwnProperty('stock_quantity_live')) {
				cartItem.stock_quantity_live = cartItem.stock_quantity_live++;
			}

			localStorage.setItem('cart_items', JSON.stringify(this.items));
		},

		deleteProductFromCart(id) {
			console.log('deleteProductFromCart', id);
			let indexItem = this.items.findIndex((item) => item.id == id);
			if (indexItem > -1) {
				this.items.splice(indexItem, 1);
				localStorage.setItem('cart_items', JSON.stringify(this.items));
			}
		},

		setCartItems({ items }) {
			this.items = items;
			if (process.client) {
				if (items !== null && items.length > 0) localStorage.setItem('cart_items', JSON.stringify(this.items));
				else localStorage.removeItem('cart_items');
			}
		},

		setCheckoutStatus(status) {
			this.checkoutStatus = status;
			if (status !== null) localStorage.setItem('cart_status', status);
			else localStorage.removeItem('cart_status');
		},

		setCheckoutId(id) {
			this.checkoutId = id;
			if (id !== null) localStorage.setItem('cart_id', id);
			else localStorage.removeItem('cart_id');
		},
		
		// from NuxtShop
		toggleCart(state?: boolean) {
			if (typeof state === "boolean") {
				this.cartOpen = state;
			} else {
				this.cartOpen = !this.cartOpen;
			}
		},
		
		async cartCreate() {
		  try {
			this.loading = true;
			
			const { $directus } = useNuxtApp();
			
			// save cart in special endpoint
			let requestResult = await useCustomFetch('/cart', {
				method: 'POST', //button.action.method,
				body: formData
			});
			
			/*
			// graphql-tag
			cartCreate {
				
				cart {
					...cart
					// cartFields
					attributes {
					  key
					  value
					}
					checkoutUrl
					createdAt
					discountCodes {
					  applicable
					  code
					}
					estimatedCost {
					  totalAmount {
						amount
						currencyCode
					  }
					  subtotalAmount {
						amount
						currencyCode
					  }
					  totalTaxAmount {
						amount
						currencyCode
					  }
					  totalDutyAmount {
						amount
						currencyCode
					  }
					}
					id
					
					// cartLines
					lines(first: 50) {
					  edges {
						cursor
						node {
						  attributes {
							key
							value
						  }
						  discountAllocations {
							discountedAmount {
							  amount
							  currencyCode
							}
						  }
						  estimatedCost {
							subtotalAmount {
							  amount
							  currencyCode
							}
							totalAmount {
							  amount
							  currencyCode
							}
						  }
						  id
						  merchandise {
							... on ProductVariant {
							  availableForSale
							  compareAtPriceV2 {
								amount
								currencyCode
							  }
							  currentlyNotInStock
							  id
							  priceV2 {
								amount
								currencyCode
							  }
							  product {
								handle
								id
								featuredImage {
								  ... on Image {
									thumbnail: url(transform: { maxWidth: 160, maxHeight: 160 })
								  }
								  altText
								  height
								  width
								}
								tags
								title
							  }
							  selectedOptions {
								name
								value
							  }
							}
						  }
						  quantity
						}
					  }
					}
		
				}
				
				userErrors {
					code
					field
					message
				}
			}	
			*/
							
			//const { resolveClient } = useApolloClient();
			//const apolloClient = resolveClient();
			
			const { data } = await apolloClient.mutate({
			  mutation: cartCreate,
			});
			if (!data.cartCreate.cart.id) {
			  throw "cartCreate: error";
			}
			this.cart.id = data?.cartCreate?.cart?.id ?? null;
			this.cart.checkoutUrl = data?.cartCreate?.cart?.checkoutUrl ?? null;
		  } catch (e) {
			return e;
		  } finally {
			this.loading = false;
		  }
		},
		
		async getCheckoutUrl() {
		  if (this.checkoutUrl) {
			return this.checkoutUrl;
		  }
		  if (!this.cart.id) {
			await this.cartCreate();
		  }
		  try {
			this.loading = true;
			const { resolveClient } = useApolloClient();
			const apolloClient = resolveClient();
			const { data } = await apolloClient.query({
			  query: checkoutUrl,
			  variables: {
				cartId: this.cart.id,
			  },
			});
			if (!data.checkoutUrl.cart.checkoutUrl) {
			  throw "getCheckoutUrl: error";
			}
			this.cart.checkoutUrl = data?.checkoutUrl?.cart?.checkoutUrl ?? null;
			return this.cart.checkoutUrl;
		  } catch (e) {
			return e;
		  } finally {
			this.loading = false;
		  }
		},
		
		async cartLinesAdd(merchandiseId: string) {
		  try {
			if (!this.cart.id) {
			  await this.cartCreate();
			}
			const { resolveClient } = useApolloClient();
			const apolloClient = resolveClient();
			const { data } = await apolloClient.mutate({
			  mutation: cartLinesAdd,
			  variables: {
				lines: [
				  {
					merchandiseId,
				  },
				],
				cartId: this.cart.id,
			  },
			});
			if (!data.cartLinesAdd) {
			  throw "cartLinesAdd: error";
			}
			this.cart = data?.cartLinesAdd?.cart;
			this.cartOpen = true;
		  } catch (e) {
			return e;
		  } finally {
			this.loading = false;
		  }
		},
		async cartLinesRemove(merchandiseId: string) {
		  try {
			const { resolveClient } = useApolloClient();
			const apolloClient = resolveClient();
			const cartId = this.cart.id;
			if (!cartId) {
			  throw "cartLinesRemove: no cartId";
			}
			const { data } = await apolloClient.mutate({
			  mutation: cartLinesRemove,
			  variables: {
				lineIds: [merchandiseId],
				cartId,
			  },
			});
			if (!data.cartLinesRemove) {
			  throw "cartLinesRemove: error";
			}
			this.cart = data.cartLinesRemove.cart;
			this.cartOpen = true;
		  } catch (e) {
			return e;
		  } finally {
			this.loading = false;
		  }
		},

  },
  	hydrate(state, initialState) {
		// https://pinia.vuejs.org/cookbook/composables.html
		// in this case we can completely ignore the initial state since we want to read the value from the browser
		//state.user = useLocalStorage('pinia/auth/login', 'bob')
		
		const config = useRuntimeConfig();
		console.log('cart config from hydrate', config, cart_default_currency_code);

		//state.token = (typeof localStorage !== 'undefined' && localStorage.getItem('auth_token')) || null; // retrieve access_token
		//state.user = (typeof localStorage !== 'undefined' && localStorage.getItem('user_info') && JSON.parse(localStorage.getItem('user_info'))) || null; // retrieve user_info

		console.warn('___cartStore - init hydrate', initialState, state, process.client);
	},	
	persist: {
		key: 'storeCart',
		//storage: persistedState.localStorage // persistedState.sessionStorage,
		storage: persistedState.cookiesWithOptions({
			sameSite: 'strict',
		}),
	}	
});

if (import.meta.hot) {
  //import.meta.hot.accept(acceptHMRUpdateWithHydration(useStore, import.meta.hot))
  import.meta.hot.accept(acceptHMRUpdate(useCartStore, import.meta.hot))
}
