//import { defineStore } from "pinia"; // no need with auto import on nuxt-dynamic-layer
import { unref } from 'vue';

export const useDynamicStore = defineStore('dynamic', {
	// arrow function recommended for full type inference
	state: () => {
		return {
			// all these properties will have their type inferred automatically
			counter: 0,
			name: 'Eduardo',
		  appContext: null, // app & theme
		  pages: {}, // pages data in cache
		  activeSlugKey: '', // current active dynamic slug only
		  previousSlugKey: '', // previous dynamic slug only
		  page: {}, // current page object without _metadata & query/params
		  layout: 'blank', // current active layout
		  // @todo add more layout options in shared
		  loader: false, // dynamic loader state
		  skeleton: false, // skeleton mode
		};
	},
	// optional getters
	getters: {
		// getters receive the state as first parameter
		doubleCounter: (state) => state.counter * 2,
		// use getters in other getters
		doubleCounterPlusOne(): number {
			return this.doubleCounter + 1;
		},
		hasPageByKey: (state) => {
		  return (slugKey) => {
			return Object.keys(state.pages).length > 0 && state.pages.hasOwnProperty(slugKey);
		  }
		},
		getPageByKey: (state) => {
		  return (slugKey) => state.pages[slugKey];
		},
		/*
		getUserById: (state) => {
		  return (userId) => state.users.find((user) => user.id === userId);
		},
		*/
	},
	// optional actions (`this` is the store instance)
	actions: {
		reset() {
			this.counter = 0;
		},
		setLoader(_loader: boolean) {
		  // manage layout loader
		  this.loader = _loader;
		},
		setLayout(layout_template: string) {
		  if (layout_template != this.layout) {
			this.layout = layout_template;
			console.log('📓 [dynamicStore] write layout', this.layout);
		  } else {
			console.log('📔 [dynamicStore] no change layout', this.layout, layout_template);
		  }
		},
		setAppContext(app_config) {
		  this.appContext = unref(app_config);
		  //this.appContext = app_config;
		  console.log('📙 [dynamicStore] write appContext', this.appContext);
		},
		setPageContext(slugKey, page_config) {
		  if (Object.keys(this.pages).length > 0) this.previousSlugKey = this.activeSlugKey;
		  this.pages[slugKey] = unref(page_config);
		  this.activeSlugKey = slugKey;
		  this.page = this.pages[slugKey].page || {};
		  console.log('📙 [dynamicStore] write page '+slugKey);
		},
		resetCache() {
		  this.appContext = null;
		  this.pages = {};
		  this.page = {};
		  this.activeSlugKey = '';
		  this.previousSlugKey = '';
		  this.layout = 'blank';
		},
	},
	hydrate(state, initialState) {
		// in this case we can completely ignore the initial state since we
		// want to read the value from the browser
		//state.user = useLocalStorage('pinia/auth/login', 'bob')
		console.warn('useDynamicStore - init', initialState);
		//console.warn('useDynamicStore - init', JSON.stringify(initialState));
		//state.counter = mainStoreInitialData.counter
	},
	persist: false,
	/*
	persist: {
		//persistedState
		// store options goes here
		key: 'storeDynamic',
		debug: true,
		//persist: false,
		//storage: window.sessionStorage,
		storage: persistedState.cookiesWithOptions({
			sameSite: 'strict',
		}),
		//overwrite: true, // on initial step
		// serialize: (value) => stringify(value), // use here to encrypt LZ-UTF8
		// deserialize // use here to decrypt LZ-UTF8
	},
	*/
});

if (import.meta.hot) {
	import.meta.hot.accept(acceptHMRUpdate(useDynamicStore, import.meta.hot));
}
