// import { defineStore } from "pinia"; // no need with auto import on nuxt-dynamic-layer
//import { directus } from '@/services/directus';
//import { LOCATION_TYPE } from '@/constants';

const listStore_lstItems = (typeof localStorage !== 'undefined' && localStorage.getItem('listStore_lstItems') && JSON.parse(localStorage.getItem('listStore_lstItems'))) || {};
const listStore_defaultOptions = (typeof localStorage !== 'undefined' && localStorage.getItem('listStore_defaultOptions') && JSON.parse(localStorage.getItem('listStore_defaultOptions'))) || {};
const listStore_selectItems = (typeof localStorage !== 'undefined' && localStorage.getItem('listStore_selectItems') && JSON.parse(localStorage.getItem('listStore_selectItems'))) || {};
const listStore_selectItemsByName = (typeof localStorage !== 'undefined' && localStorage.getItem('listStore_selectItemsByName') && JSON.parse(localStorage.getItem('listStore_selectItemsByName'))) || {};

export const useListStore = defineStore("list", {
  // id: 'listStore',
  state: () => {
    return {
		lstItems: listStore_lstItems, // { },
		defaultOptions: listStore_defaultOptions, // { },
		selectItems: listStore_selectItems, // { },
		selectItemsByName: listStore_selectItemsByName, // { },
    };
  },
  getters: {
		customDefaultOptions(state) {
			return (collection, options = {}) => {
				if (!options.hasOwnProperty('keyLabel')) options.keyLabel = 'name';
				if (!options.hasOwnProperty('keyId')) options.keyId = 'id';
				if (!options.hasOwnProperty('extraFieldKey')) options.extraFieldKey = null;
				if (!options.hasOwnProperty('extraFieldLabel')) options.extraFieldLabel = null;
				if (!options.hasOwnProperty('keyActive')) options.keyActive = 'active';
				if (!options.hasOwnProperty('fieldActive')) options.fieldActive = null;

				let defaultOptions = [];
				state.lstItems[collection].forEach(function (elem) {
					let tmpLabel = elem[options.keyLabel];
					if (options.extraFieldKey && elem[options.keyLabel] != null) {
						tmpLabel = elem[options.keyLabel]+' ('+elem[options.extraFieldKey];
						if (options.extraFieldLabel) tmpLabel += ' '+options.extraFieldLabel;
						tmpLabel += ')';
					}
					if (options.fieldActive !== null) {
						if (elem[options.keyActive] !== options.fieldActive) return;
					}

					defaultOptions.push({
						value: elem[options.keyId],
						label: tmpLabel,
						data: elem
					});
				});

				return defaultOptions;
			}
		}
  },
  actions: {
		// @todo add forceRefreshing and call refreshList when change admin
		async refreshList() {
			let forceRefreshing = false;

			// @todo put delta time on config variable
			if (typeof window !== 'undefined' && typeof window.localStorage !== 'undefined' && window.localStorage['listStore_lastUpdate'] != undefined && window.localStorage['listStore_lastUpdate'] != '') {
				let deltaPersistentTime = new Date().getTime() - parseInt(window.localStorage['listStore_lastUpdate']);
				if (deltaPersistentTime > 14400000) {
					forceRefreshing = true; // 4 hours
					console.log('listStore - refreshList - forceRefreshing');
				} else {
					console.log('listStore - refreshList - use local cache');
				}
			}

			if (forceRefreshing || !this.lstItems.hasOwnProperty('suppliers') || this.lstItems['suppliers'].length == 0) await this.loadCollection('suppliers');
			if (forceRefreshing || !this.lstItems.hasOwnProperty('categories') || this.lstItems['categories'].length == 0) await this.loadCollection('categories', { sort: 'sort', fieldActive: 1 });
			if (forceRefreshing || !this.lstItems.hasOwnProperty('committees') || this.lstItems['committees'].length == 0) await this.loadCollection('committees');
			if (forceRefreshing || !this.lstItems.hasOwnProperty('docks') || this.lstItems['docks'].length == 0) await this.loadCollection('docks');
			if (forceRefreshing || !this.lstItems.hasOwnProperty('locations') || this.lstItems['locations'].length == 0) await this.loadCollection('locations', { sort: 'location', keyLabel: 'location', keyId: 'location', fieldActive: 1 });
		},

		// collection: suppliers, categories (sort='sort', fieldActive=1), committees, docks, locations (keyId=location, keyLabel=location)
		async loadCollection(collection, options = {}) {
			var self = this;

			//let options = options || {};
			if (!options.hasOwnProperty('fields')) options.fields = '*';
			if (!options.hasOwnProperty('sort')) options.sort = 'name'; // name or sort
			if (!options.hasOwnProperty('keyLabel')) options.keyLabel = 'name';
			if (!options.hasOwnProperty('keyId')) options.keyId = 'id';
			if (!options.hasOwnProperty('keyActive')) options.keyActive = 'active';
			if (!options.hasOwnProperty('fieldActive')) options.fieldActive = null;
			if (!options.hasOwnProperty('saveLocalStorage')) options.saveLocalStorage = true;
						
			// include extend constants list
			if (!options.hasOwnProperty('constants')) options.constants = null;
			/*
			const LOCATION_TYPE = {
				"reception": "Réception",
				"stockage": "Stockage",
				"preparation": "Préparation",
				"expedition": "Expédition",
			}
			*/

			console.log('___listStore - load '+collection+' - start', options);

			const { $directus } = useNuxtApp();

			//setTimeout(async function() {
			let queryOptions = {
				fields: options.fields,
				filter: {},
				sort: options.sort,
				limit: -1,
			};

			if (options.fieldActive !== null) queryOptions.filter[options.keyActive] = { _eq: options.fieldActive };

			const response = await $directus.items(collection).readByQuery(queryOptions).catch((err) => {
				console.error('___listStore - load '+collection+' - error', err);
			});

			if (response && response.data && response.data.length > 0) {
				this.lstItems[collection] = response.data;

				this.defaultOptions[collection] = [];
				this.selectItems[collection] = {};
				this.selectItemsByName[collection] = {};
				this.lstItems[collection].forEach(function (elem) {
					let tmpLabel = elem[options.keyLabel];
					// if (collection == 'locations') tmpLabel = elem[options.keyLabel]+' ('+LOCATION_TYPE[elem.type]+')';
					if (options.constants && Object.keys(options.constants).length > 0) tmpLabel = elem[options.keyLabel]+' ('+options.constants[elem.type]+')';

					self.selectItems[collection][elem[options.keyId]] = tmpLabel;
					self.selectItemsByName[collection][elem[options.keyLabel]] = tmpLabel;

					self.defaultOptions[collection].push({
						value: elem[options.keyId],
						label: tmpLabel,
						data: elem
					});
				});

				if (options.saveLocalStorage) {
					localStorage.setItem('listStore_lstItems', JSON.stringify(this.lstItems));
					localStorage.setItem('listStore_defaultOptions', JSON.stringify(this.defaultOptions));
					localStorage.setItem('listStore_selectItems', JSON.stringify(this.selectItems));
					localStorage.setItem('listStore_selectItemsByName', JSON.stringify(this.selectItemsByName));
					localStorage.setItem('listStore_lastUpdate', new Date().getTime());
				}

				console.log('___listStore - load '+collection+' - end', this.lstItems[collection], this.defaultOptions[collection]);
			}
			//}, 0);
		},
  },
})

if (import.meta.hot) {
  import.meta.hot.accept(acceptHMRUpdate(useListStore, import.meta.hot))
}